/***************************************************************************
                          sglwindow.cpp  -  description
                             -------------------
    begin                : Wed Jan 23 2002
    copyright            : (C) 2002 by Andre Stein
    email                : andre_stein@web.de
    url                  : www.steinsoft.net    
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "sglwindow.h"
#include <SDL.h>

const float WHOLE_SCENE_LIGHT[] = { 0.5f, 0.5f, 0.5f, 1.0f};
const float DIFFUSE_LIGHT[]  = { 0.6f, 0.6f, 0.6f, 1.0f};
const float SPECULAR_LIGHT[] = {-.1f, -0.1f, -0.1f, 1.0f};
const float LIGHT_POSITION1[] = { 0.0f, -5.0f,0.0f, 1.0f};
const float LIGHT_POSITION2[] = { 0.0f, 5.0f,0.0f, 1.0f};


const float SPOT_DIRECTION[] = {0.0f,-1.0f,-1.0f};

const float MATERIAL[] = {1.0f,1.0f,1.0f,1.0f};

SGLWindow::SGLWindow()
: showWater(true)
{
}

SGLWindow::~SGLWindow()
{
}
/** No descriptions */
void SGLWindow::init()
{
	glClearColor(0.0f,0.0f,0.0f,0.0f);
	glShadeModel(GL_SMOOTH);
	glClearDepth(1.0f);
	glEnable(GL_DEPTH_TEST);
	glDepthFunc(GL_LEQUAL);
	glHint(GL_PERSPECTIVE_CORRECTION_HINT, GL_NICEST);
	glEnable(GL_LINE_SMOOTH);
	
	glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
	
	/************
	 * LIGHT
	 **************/
	 
	glLightModelfv(GL_LIGHT_MODEL_AMBIENT,WHOLE_SCENE_LIGHT);

	//Main
	glLightfv(GL_LIGHT0,GL_AMBIENT,SPECULAR_LIGHT);
	glLightfv(GL_LIGHT0,GL_SPECULAR,SPECULAR_LIGHT);
	glLightfv(GL_LIGHT0,GL_DIFFUSE,DIFFUSE_LIGHT);
	glLightfv(GL_LIGHT0,GL_POSITION,LIGHT_POSITION1);

	glLightfv(GL_LIGHT1,GL_AMBIENT,SPECULAR_LIGHT);
	glLightfv(GL_LIGHT1,GL_SPECULAR,SPECULAR_LIGHT);
	glLightfv(GL_LIGHT1,GL_DIFFUSE,DIFFUSE_LIGHT);
	glLightfv(GL_LIGHT1,GL_POSITION,LIGHT_POSITION2);

	//Spotlight
	glLightf(GL_LIGHT0, GL_SPOT_CUTOFF, 10.0f); //60 Spotlight
	glLightf(GL_LIGHT0,GL_SPOT_EXPONENT, 60.0f);
	glLightfv(GL_LIGHT0,GL_SPOT_DIRECTION,SPOT_DIRECTION);

    	glColorMaterial(GL_FRONT,GL_AMBIENT_AND_DIFFUSE);

	//Materials
	glMaterialfv(GL_FRONT,GL_SPECULAR,MATERIAL);
	glMaterialf(GL_FRONT,GL_SHININESS,32.0f);

	glEnable(GL_COLOR_MATERIAL);
	glEnable(GL_LIGHTING);
	glEnable(GL_LIGHT0);
	glEnable(GL_LIGHT1);
	
	
	//init water wave
	water.init();
}
/** No descriptions */
void SGLWindow::resize(int width, int height)
{
	glViewport(0,0,width,height);

	if (height == 0)
		height = 1;	

	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();

	gluPerspective(45.0f,width/height,0.1f,1000.0f);
	//gluOrtho2D(-6.0f,6.0f,3.0f,-3.0f);
	
	glMatrixMode(GL_MODELVIEW);
	glLoadIdentity();
}
/** No descriptions */
void SGLWindow::update(float frametime)
{
	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);

	glLoadIdentity();
	
	//depending on 'showWater', we either just draw the rope
	//or the water
	if (!showWater)
	{
		//draw our rope..
		rope.update(frametime);
	}
	else
	{
		//update water
		water.update(frametime);
		
// 		//water needs da mouse..
		handleMouse();
	}
}

void SGLWindow::keyDown(int key)
{
	switch(key)
 	{
    	case SDLK_ESCAPE:
     		SDL_Event sent;
       		sent.type = SDL_QUIT;

         	SDL_PushEvent(&sent);
          	break;
	case SDLK_SPACE:
		//switch rope/water animation
		showWater = !showWater;
		break;
        }
}

void SGLWindow::handleMouse()
{
	int x,y;
	
	//get relative mouse position
	SDL_GetRelativeMouseState(&x,&y);


	//calculate movement depending on displacement from
	//inital mouse pos
	float moveX = float(x)/10.0f;
		
	//move camera on water wave by that
	// (but only if activated)
	if (showWater)
	{
		water.moveCamera(moveX);
	}
	
	
}
