/***************************************************************************
 *   Copyright (C) 2003 by Andr Stein                                     *
 *   stone@steinsoft.net                                                   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 ***************************************************************************/
#include "sropewave.h"
#include <GL/gl.h>

#define INIT_X -5.0f
#define INIT_Y 0.0f
#define ROPE_WIDTH 10.0f
#define STEP_SIZE 0.01f
#define INIT_SPEED 0.0005f
#define DECREASE_SPEED 0.0001f
#define INIT_AMPLITUDE 1.5f

const float LINE_COLOR[] = { 0.0f, 0.75f , 0.0f };

SRopeWave::SRopeWave()
{
	wave.amplitude = INIT_AMPLITUDE;
	wave.lambda = 1.0f;
	wave.setSpeed(INIT_SPEED);
}

SRopeWave::~SRopeWave()
{
}

void SRopeWave::update(float frametime)
{
	wave.time += frametime;
	
	//amplitude is getting smaller
	// using differential equation
	// N(t) = N(0)*e^(-factor*t)
	//          ^ starting amplitude
	
	wave.amplitude = INIT_AMPLITUDE*pow(M_E,-DECREASE_SPEED*wave.time);
		
	draw();
}

void SRopeWave::draw()
{
	float x,y;

	glPushMatrix();
	
	glTranslatef(INIT_X, INIT_Y, -15.0f);
	glColor3fv(LINE_COLOR);
	
	/**
	 * create rope wave one by one..
	 *  made out of lines
	 **/
	
	glBegin(GL_LINE_STRIP);
	 
	for (x = 0.0f; x <= ROPE_WIDTH; x += STEP_SIZE)
	{
		y = wave.y(x);
		
		//draw vertex of line
		glVertex2f( x, y );
	}
	
	glEnd();

	glPopMatrix();
}

