/***************************************************************************
 *   Copyright (C) 2003 by Andr Stein                                     *
 *   stone@steinsoft.net                                                   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 ***************************************************************************/
#include "swaterwave.h"
#include <iostream>
#include <math.h>

using namespace std;

#define INIT_SPEED 0.001f
#define DECREASE_SPEED 0.0001f
#define INIT_AMPLITUDE 0.5f

const double TO_RAD = (2*M_PI)/360;

SWaterWave::SWaterWave()
:cameraRotation(0.0f)
{
}


SWaterWave::~SWaterWave()
{
}

void SWaterWave::init()
{
	waterTexture.createTexture("data/water.bmp");
	
	/**
	 * init water
	 ****/
	for (int x = 0; x < WATER_X_SIZE; x++)
	{
		for (int z = 0; z < WATER_Z_SIZE; ++z)
		{
			waterHeight[x][z] = 0.0f;
		}
	}
	
	/**
	 * init wave
	 ***/
	 
	wave[0].amplitude = INIT_AMPLITUDE;
	wave[0].lambda = 2.0f;
	wave[0].setSpeed(INIT_SPEED);
	
	wave[1].amplitude = -INIT_AMPLITUDE;
	wave[1].lambda = 3.0f;
	wave[1].setSpeed(INIT_SPEED);
}

void SWaterWave::draw()
{
	glPushMatrix();
	
//	cout << cameraRotation << endl;
	
	//do some camera adjustements

	//we calculate the position of the camera using
	//the rotation specified.. 
	//the y-value will always be at the same level
	//while x/z are calculated using cos and sin
	//  notice: sin/cos accept degrees in radians
	gluLookAt((-WATER_X_SIZE/2) * cos(cameraRotation*TO_RAD),
		  35.0f,
		  -40.0f * sin(cameraRotation*TO_RAD),
		  32.0f,
		  0.0f,
		  32.0f,
		  0.0f,
		  1.0f,
		  0.0f);
	
	/**
	 * iterate through water heights and create
	 * textured quads which are the water..
	 **/
	 
	glEnable(GL_TEXTURE_2D);
	glEnable(GL_CULL_FACE);
	glEnable(GL_BLEND);
	
	waterTexture.select();
	 
	glBegin(GL_QUADS);
	glColor4f(1.0f,1.0f,1.0f,0.5f);
	 
	for (float z = 0; z < WATER_Z_SIZE-1; z += 1.0f)
	{	
		for (float x = 0; x < WATER_X_SIZE-1; x += 1.0f)
		{
			glNormal3f(0.0f,1.0f,0.0f);
			glTexCoord2f((x*5)/WATER_X_SIZE, (z*5)/WATER_Z_SIZE);
			glVertex3f(x,waterHeight[int(x)][int(z)],z);
			
			glNormal3f(0.0f,1.0f,0.0f);
			glTexCoord2f((x*5)/WATER_X_SIZE, ((z+1)*5)/WATER_Z_SIZE);
			glVertex3f(x,waterHeight[int(x)][int(z)+1],z+1);
			
			glNormal3f(0.0f,1.0f,0.0f);
			glTexCoord2f(((x+1)*5)/WATER_X_SIZE, ((z+1)*5)/WATER_Z_SIZE);
			glVertex3f(x+1,waterHeight[int(x)+1][int(z)+1],z+1);
			
			glNormal3f(0.0f,1.0f,0.0f);
			glTexCoord2f( ((x+1)*5)/WATER_X_SIZE, (z*5)/WATER_Z_SIZE);
			glVertex3f(x+1,waterHeight[int(x)+1][int(z)],z);
		}
	}
	
	glEnd();
	
	glDisable(GL_TEXTURE_2D);
	glDisable(GL_CULL_FACE);
	glDisable(GL_BLEND);
	
	glPopMatrix();
}

void SWaterWave::update(float frametime)
{
	/***
	 * wave 1
	 ***/

	 wave[0].time += frametime;
	 wave[1].time += frametime;
	 
	//update all height-positions of the points
	//  -> waves are collisijng (interference) so 
	//  updated poisitons are added together
	//wave 0 coming from top (z-axis)
	//wave 1 coming from the left (x-axis)
	for (int z = 0; z < WATER_Z_SIZE; ++z)
	{
		for (int x = 0; x < WATER_X_SIZE; ++x)
		{
			waterHeight[x][z] = wave[0].y(z)+wave[1].y(x);
		}
	}
	
	
	//finally draw...
	draw();
}

void SWaterWave::moveCamera(float by)
{
	cameraRotation += by;
	
	if (cameraRotation > 360.0f)
	{
		cameraRotation = cameraRotation - 360.0f;
	}
	else if (cameraRotation < 0.0f)
	{
		cameraRotation = 360.0f + cameraRotation;
	}
		
}
